@@ -1,6 +1,6 @@
 /* QuesoGLC
  * A free implementation of the OpenGL Character Renderer (GLC)
- * Copyright (c) 2002, 2004-2007, Bertrand Coconnier
+ * Copyright (c) 2002, 2004-2008, Bertrand Coconnier
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU Lesser General Public
@@ -16,7 +16,7 @@
  *  License along with this library; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
-/* $Id: context.c,v 1.78 2007/10/20 23:02:11 bcoconni Exp $ */
+/* $Id: context.c 841 2008-10-19 10:36:00Z bcoconni $ */
 
 /** \file
  * defines the so-called "Context commands" described in chapter 3.5 of the GLC
@@ -44,8 +44,9 @@
  * undefined.
  */
 
+#include <string.h>
+
 #include "internal.h"
-#include "oglyph.h"
 
 
 
@@ -182,6 +183,12 @@ void APIENTRY glcDeleteGLObjects(void)
     glDeleteBuffersARB(1, &ctx->texture.bufferObjectID);
     ctx->texture.bufferObjectID = 0;
   }
+
+  /* Delete the vertex buffer object */
+  if (GLEW_ARB_vertex_buffer_object && ctx->atlas.bufferObjectID) {
+    glDeleteBuffersARB(1, &ctx->atlas.bufferObjectID);
+    ctx->atlas.bufferObjectID = 0;
+  }
 }
 
 
@@ -228,8 +235,8 @@ static void __glcChangeState(GLCenum inAttrib, GLboolean value)
   case GLC_MIPMAP:
     ctx->enableState.mipmap = value;
     if (ctx->atlas.id) {
-      GLint boundTexture = 0;
-      glGetIntegerv(GL_TEXTURE_BINDING_2D, &boundTexture);
+      GLuint boundTexture = 0;
+      glGetIntegerv(GL_TEXTURE_BINDING_2D, (GLint*)&boundTexture);
       glBindTexture(GL_TEXTURE_2D, ctx->atlas.id);
       if (ctx->enableState.mipmap)
         glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER,
@@ -312,17 +319,18 @@ void APIENTRY glcDisable(GLCenum inAttrib)
  *  - \b GLC_GL_OBJECTS : if enabled, GLC stores characters rendering commands
  *    in GL display lists and textures (if any) in GL texture objects.
  *  - \b GLC_MIPMAP : if enabled, texture objects used by GLC are mipmapped
- *  - \b GLC_HINTING_QSO : if enabled, GLC uses the auto-hinting procedures
- *    that are available for most scalable fonts. It gives better results for
- *    characters that are rendered at small sizes. Hinting may however generate
- *    visual artifacts such as shaking outlines if the character is animated.
- *    This attribute should be disabled in such cases.
+ *  - \b GLC_HINTING_QSO : if enabled, GLC uses the hinting procedures that are
+ *    available for most scalable fonts. It gives better results for characters
+ *    that are rendered at small sizes. This attribute is ignored when
+ *    \b GLC_GL_OBJECTS is enabled. Hinting may generate visual artifacts such
+ *    as "shaking outlines" if the character is animated. This attribute should
+ *    be disabled in such cases.
  *  - \b GLC_EXTRUDE_QSO : if enabled and \b GLC_RENDER_STYLE is
  *    \b GLC_TRIANGLE then GLC renders extruded characters with a thickness
  *    equal to 1.0. A call to glScale3*(1., 1., \e thickness ) can be added
  *    before the rendering commands in order to obtain the desired thickness.
- *  - \b GLC_KERNING_QSO : if enabled, GLC uses kerning information (when
- *    available) when string are rendered or measured.
+ *  - \b GLC_KERNING_QSO : if enabled, GLC uses kerning information when
+ *    rendering or measuring a string. Not all fonts have kerning informations.
  *
  *  \param inAttrib A symbolic constant indicating a GLC attribute.
  *  \sa glcDisable()
@@ -406,7 +414,7 @@ const GLCchar* APIENTRY glcGetListc(GLCenum inAttrib, GLint inIndex)
   __GLCcontext *ctx = NULL;
   GLCchar8* catalog = NULL;
   GLCchar* buffer = NULL;
-  int length = 0;
+  size_t length = 0;
 
   GLC_INIT_THREAD();
 
@@ -504,6 +512,12 @@ const GLCchar* APIENTRY glcGetListc(GLCenum inAttrib, GLint inIndex)
  *      <td>\<empty list\></td>
  *      <td><b>GLC_TEXTURE_OBJECT_COUNT</b></td>
  *    </tr>
+ *    <tr>
+ *      <td><b>GLC_BUFFER_OBJECT_LIST_QSO</b></td>
+ *      <td>0x800F</td>
+ *      <td>\<empty list\></td>
+ *      <td><b>GLC_BUFFER_OBJECT_COUNT_QSO</b></td>
+ *    </tr>
  *  </table>
  *  </center>
  *
@@ -529,6 +543,16 @@ GLint APIENTRY glcGetListi(GLCenum inAttrib, GLint inIndex)
   case GLC_LIST_OBJECT_LIST:
   case GLC_TEXTURE_OBJECT_LIST:
     break;
+  case GLC_BUFFER_OBJECT_LIST_QSO: /* QuesoGLC extension */
+    /* This parameter is available only if the corresponding GL extensions are
+     * supported by the GL driver.
+     */
+    if (GLEW_ARB_vertex_buffer_object || GLEW_ARB_pixel_buffer_object)
+      break;
+    else {
+      __glcRaiseError(GLC_PARAMETER_ERROR);
+      return 0;
+    }
   default:
     __glcRaiseError(GLC_PARAMETER_ERROR);
     return 0;
@@ -562,20 +586,16 @@ GLint APIENTRY glcGetListi(GLCenum inAttrib, GLint inIndex)
 
     if (node)
       return ((__GLCfont*)node->data)->id;
-    else {
-      __glcRaiseError(GLC_PARAMETER_ERROR);
-      return 0;
-    }
+    else
+      break;
   case GLC_FONT_LIST:
     for (node = ctx->fontList.head; inIndex && node;
          node = node->next, inIndex--);
 
     if (node)
       return ((__GLCfont*)node->data)->id;
-    else {
-      __glcRaiseError(GLC_PARAMETER_ERROR);
-      return 0;
-    }
+    else
+      break;
   case GLC_LIST_OBJECT_LIST:
     /* In order to get the display list name, we have to perform a search
      * through the list of display lists of every face descriptor.
@@ -596,8 +616,7 @@ GLint APIENTRY glcGetListi(GLCenum inAttrib, GLint inIndex)
 	  inIndex -= count;
       }
     }
-    __glcRaiseError(GLC_PARAMETER_ERROR);
-    return 0;
+    break;
   case GLC_TEXTURE_OBJECT_LIST:
     switch(inIndex) {
       /* QuesoGLC uses at most 2 textures : one for immediate mode rendering and
@@ -624,10 +643,60 @@ GLint APIENTRY glcGetListi(GLCenum inAttrib, GLint inIndex)
       default:
 	break;
     }
-    __glcRaiseError(GLC_PARAMETER_ERROR);
-    return 0;
+    break;
+  case GLC_BUFFER_OBJECT_LIST_QSO: /* QuesoGLC extension */
+    switch(inIndex) {
+      /* QuesoGLC uses at most 2 buffer objects : one PBO for immediate mode
+       * rendering and one VBO for the texture atlas. That's all. They are
+       * virtually stored in the following order : PBO for immediate mode first,
+       * then VBO for texture atlas.
+       */
+      case 0:
+	if (ctx->texture.bufferObjectID)
+	  return ctx->texture.bufferObjectID;
+	/* If the PBO for immediate mode does not exist, then the first buffer
+	 * object is the VBO for the texture atlas.
+	 * NOTE: if the texture atlas is created first and the PBO for
+	 * immediate mode is created after then this algorithm leads to a
+	 * modification of the order in which buffer objects are reported which
+	 * is not satisfying...
+	 */
+	if (ctx->atlas.bufferObjectID)
+	  return ctx->atlas.bufferObjectID;
+	break;
+      case 1:
+	if ((ctx->texture.bufferObjectID) && (ctx->atlas.bufferObjectID))
+	  return ctx->atlas.bufferObjectID;
+	break;
+      default:
+	break;
+    }
+
+    if (ctx->texture.bufferObjectID)
+      inIndex--;
+    if (ctx->atlas.bufferObjectID)
+      inIndex--;
+
+    for (node = ctx->fontList.head; node; node = node->next) {
+      __GLCfaceDescriptor* faceDesc =
+		(__GLCfaceDescriptor*)(((__GLCfont*)(node->data))->faceDesc);
+      FT_ListNode glyphNode = NULL;
+
+      for (glyphNode = faceDesc->glyphList.head; glyphNode;
+	   glyphNode = glyphNode->next) {
+	__GLCglyph* glyph = (__GLCglyph*)glyphNode;
+	int count = __glcGlyphGetBufferObjectCount(glyph);
+
+	if (inIndex < count)
+	  return __glcGlyphGetBufferObject(glyph, inIndex);
+	else
+	  inIndex -= count;
+      }
+    }
+    break;
   }
 
+  __glcRaiseError(GLC_PARAMETER_ERROR);
   return 0;
 }
 
@@ -706,14 +775,15 @@ GLvoid* APIENTRY glcGetPointer(GLCenum inAttrib)
  */
 const GLCchar* APIENTRY glcGetc(GLCenum inAttrib)
 {
-  static GLCchar* __glcExtensions = (GLCchar*) "GLC_QSO_attrib_stack"
-    " GLC_QSO_extrude GLC_QSO_hinting GLC_QSO_kerning GLC_QSO_matrix_stack"
-    " GLC_QSO_utf8 GLC_SGI_full_name";
-  static GLCchar* __glcVendor = (GLCchar*) "The QuesoGLC Project";
+  static const char* __glcExtensions1 = "GLC_QSO_attrib_stack";
+  static const char* __glcExtensions2 = " GLC_QSO_buffer_object";
+  static const char* __glcExtensions3 = " GLC_QSO_extrude GLC_QSO_hinting"
+    " GLC_QSO_kerning GLC_QSO_matrix_stack GLC_QSO_utf8 GLC_SGI_full_name";
+  static const GLCchar8* __glcVendor = (const GLCchar8*) "The QuesoGLC Project";
 #ifdef HAVE_CONFIG_H
-  static GLCchar* __glcRelease = (GLCchar*) PACKAGE_VERSION;
+  static const GLCchar8* __glcRelease = (const GLCchar8*) PACKAGE_VERSION;
 #else
-  static GLCchar* __glcRelease = (GLCchar*) QUESOGLC_VERSION;
+  static const GLCchar8* __glcRelease = (const GLCchar8*) QUESOGLC_VERSION;
 #endif
 
   __GLCcontext *ctx = NULL;
@@ -741,10 +811,23 @@ const GLCchar* APIENTRY glcGetc(GLCenum inAttrib)
   /* Translate the string to the relevant Unicode format */
   switch(inAttrib) {
   case GLC_EXTENSIONS:
-    return __glcConvertFromUtf8ToBuffer(ctx, __glcExtensions,
-					ctx->stringState.stringType);
+    {
+      GLCchar8 __glcExtensions[256];
+
+      assert((strlen(__glcExtensions1) + strlen(__glcExtensions2)
+	      + strlen(__glcExtensions3)) <= 256);
+
+      /* Build the extensions string depending on the available GL extensions */
+      strcpy((char*)__glcExtensions, __glcExtensions1);
+      if (GLEW_ARB_vertex_buffer_object || GLEW_ARB_pixel_buffer_object)
+	strcat((char*)__glcExtensions, __glcExtensions2);
+      strcat((char*)__glcExtensions, __glcExtensions3);
+
+      return __glcConvertFromUtf8ToBuffer(ctx, __glcExtensions,
+					  ctx->stringState.stringType);
+    }
   case GLC_RELEASE:
-    return __glcConvertFromUtf8ToBuffer(ctx, (GLCchar*)__glcRelease,
+    return __glcConvertFromUtf8ToBuffer(ctx, __glcRelease,
 					ctx->stringState.stringType);
   case GLC_VENDOR:
     return __glcConvertFromUtf8ToBuffer(ctx, __glcVendor,
@@ -925,6 +1008,9 @@ GLfloat* APIENTRY glcGetfv(GLCenum inAttrib, GLfloat* outVec)
  *    <td>0x800D</td>
  *    <td>\<implementation specific\></td>
  *  </tr>
+ *  <tr>
+ *    <td><b>GLC_BUFFER_OBJECT_COUNT_QSO</b></td> <td>0x800E</td> <td>0</td>
+ *  </tr>
  *  </table>
  *  </center>
  *  \param inAttrib Attribute for which an integer variable is requested.
@@ -961,16 +1047,26 @@ GLint APIENTRY glcGeti(GLCenum inAttrib)
   case GLC_ATTRIB_STACK_DEPTH_QSO:     /* QuesoGLC extension */
   case GLC_MAX_ATTRIB_STACK_DEPTH_QSO: /* QuesoGLC extension */
     break;
+  case GLC_BUFFER_OBJECT_COUNT_QSO:    /* QuesoGLC extension */
+    /* This parameter is available only if the corresponding GL extensions are
+     * supported by the GL driver.
+     */
+    if (GLEW_ARB_vertex_buffer_object || GLEW_ARB_pixel_buffer_object)
+      break;
+    else {
+      __glcRaiseError(GLC_PARAMETER_ERROR);
+      return 0;
+    }
   default:
     __glcRaiseError(GLC_PARAMETER_ERROR);
-    return GLC_NONE;
+    return 0;
   }
 
   /* Check if the thread has a current context */
   ctx = GLC_GET_CURRENT_CONTEXT();
   if (!ctx) {
     __glcRaiseError(GLC_STATE_ERROR);
-    return GLC_NONE;
+    return 0;
   }
 
   /* Returns the requested value */
@@ -1011,8 +1107,8 @@ GLint APIENTRY glcGeti(GLCenum inAttrib)
   case GLC_STRING_TYPE:
     return ctx->stringState.stringType;
   case GLC_TEXTURE_OBJECT_COUNT:
-    count += (ctx->texture.id ? 1: 0);
-    count += (ctx->atlas.id ? 1:0);
+    count += (ctx->texture.id ? 1 : 0);
+    count += (ctx->atlas.id ? 1 : 0);
     return count;
   case GLC_VERSION_MAJOR:
     return __glcCommonArea.versionMajor;
@@ -1026,6 +1122,22 @@ GLint APIENTRY glcGeti(GLCenum inAttrib)
     return ctx->attribStackDepth;
   case GLC_MAX_ATTRIB_STACK_DEPTH_QSO: /* QuesoGLC extension */
     return GLC_MAX_ATTRIB_STACK_DEPTH;
+  case GLC_BUFFER_OBJECT_COUNT_QSO:    /* QuesoGLC extension */
+    count += (ctx->texture.bufferObjectID ? 1 : 0);
+    count += (ctx->atlas.bufferObjectID ? 1 : 0);
+    for (node = ctx->fontList.head; node; node = node->next) {
+      __GLCfaceDescriptor* faceDesc =
+		(__GLCfaceDescriptor*)(((__GLCfont*)(node->data))->faceDesc);
+      FT_ListNode glyphNode = NULL;
+
+      for (glyphNode = faceDesc->glyphList.head; glyphNode;
+	   glyphNode = glyphNode->next) {
+	__GLCglyph* glyph = (__GLCglyph*)glyphNode;
+
+	count += __glcGlyphGetBufferObjectCount(glyph);
+      }
+    }
+    return count;
   }
 
   return 0;
@@ -1184,22 +1296,22 @@ void APIENTRY glcStringType(GLCenum inStringType)
  *  <table>
  *  <caption>Group attributes</caption>
  *  <tr>
- *    <td>Group attribute</td> <td>Name</td>
+ *    <td>Group attribute</td> <td>Name</td> <td>Enumerant</td>
  *  </tr>
  *  <tr>
- *    <td>enable</td> <td><b>GLC_ENABLE_BIT_QSO</b></td>
+ *    <td>enable</td> <td><b>GLC_ENABLE_BIT_QSO</b></td> <td>0x0001</td>
  *  </tr>
  *  <tr>
- *    <td>render</td> <td><b>GLC_RENDER_BIT_QSO</b></td>
+ *    <td>render</td> <td><b>GLC_RENDER_BIT_QSO</b></td> <td>0x0002</td>
  *  </tr>
  *  <tr>
- *    <td>string</td> <td><b>GLC_STRING_BIT_QSO</b></td>
+ *    <td>string</td> <td><b>GLC_STRING_BIT_QSO</b></td> <td>0x0004</td>
  *  </tr>
  *  <tr>
- *    <td> </td> <td><b>GLC_GL_ATTRIB_BIT_QSO</b></td>
+ *    <td> </td> <td><b>GLC_GL_ATTRIB_BIT_QSO</b></td> <td>0x0008</td>
  *  </tr>
  *  <tr>
- *    <td> </td> <td><b>GLC_ALL_ATTRIBS_BIT_QSO</b></td>
+ *    <td> </td> <td><b>GLC_ALL_ATTRIBS_BIT_QSO</b></td> <td>0xFFFF</td>
  *  </tr>
  *  </table>
  *  </center>
