@@ -1,6 +1,6 @@
 /* QuesoGLC
  * A free implementation of the OpenGL Character Renderer (GLC)
- * Copyright (c) 2002, 2004-2008, Bertrand Coconnier
+ * Copyright (c) 2002, 2004-2009, Bertrand Coconnier
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU Lesser General Public
@@ -16,7 +16,7 @@
  *  License along with this library; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
-/* $Id: font.c 710 2008-01-21 20:14:17Z bcoconni $ */
+/* $Id: font.c 857 2009-01-17 12:17:31Z bcoconni $ */
 
 /** \file
  *  defines the so-called "Font commands" described in chapter 3.7 of the GLC
@@ -531,7 +531,7 @@ void APIENTRY glcFontMap(GLint inFont, GLint inCode, const GLCchar* inCharName)
     /* Remove the character from the map */
     __glcCharMapRemoveChar(font->charMap, code);
   else {
-    GLCchar* buffer = NULL;
+    GLCchar8* buffer = NULL;
     GLCulong mappedCode  = 0;
     __GLCglyph* glyph = NULL;
     GLint error = 0;
@@ -591,32 +591,44 @@ GLint APIENTRY glcGenFontID(void)
     return 0;
   }
 
-  for (node = ctx->fontList.head; node; node = node->next) {
-    font = (__GLCfont*)node->data;
+  /* Look for an ID which is not associated to an existing font */
+  for (id = 1; id <= 0x7fffffff; id++) {
+    /* Check against fonts stored in GLC_FONT_LIST */
+    for (node = ctx->fontList.head; node; node = node->next) {
+      font = (__GLCfont*)node->data;
 
-    if (font->id == id) {
-      id++;
-      if (id > 0x7fffffff) {
-	__glcRaiseError(GLC_RESOURCE_ERROR);
-	return 0;
-      }
-      node = ctx->fontList.head;
+      if (font->id == id)
+	break;
     }
+
+    /* A font in GLC_FONT_LIST has already the current ID, check for next ID */
+    if (node)
+      continue;
+
+    /* Check against fonts already generated by glcGenFontID() but not yet
+     * associated to a font of GLC_FONT_LIST.
+     */
+    for (node = ctx->genFontList.head; node; node = node->next) {
+      font = (__GLCfont*)node->data;
+
+      if (font->id == id)
+	break;
+    }
+
+    /* If the current ID is not associated then exit*/
+    if (!node)
+      break;
   }
 
-  for (node = ctx->genFontList.head; node; node = node->next) {
-    font = (__GLCfont*)node->data;
-
-    if (font->id == id) {
-      id++;
-      if (id > 0x7fffffff) {
-	__glcRaiseError(GLC_RESOURCE_ERROR);
-	return 0;
-      }
-      node = ctx->genFontList.head;
-    }
+  /* If we have scanned all possible ID values and failed to find one which is
+   * not yet associated (which is very unlikely) then exit with an error.
+   */
+  if (node) {
+    __glcRaiseError(GLC_RESOURCE_ERROR);
+    return 0;
   }
 
+  /* Create an empty font */
   node = (FT_ListNode)__glcMalloc(sizeof(FT_ListNodeRec));
   if (!node) {
     __glcRaiseError(GLC_RESOURCE_ERROR);
@@ -719,13 +731,14 @@ const GLCchar* APIENTRY glcGetFontListc(GLint inFont, GLCenum inAttrib,
 {
   __GLCfont* font = NULL;
   __GLCcontext* ctx = NULL;
+  const GLCchar8* name = NULL;
 
   GLC_INIT_THREAD();
 
   /* Check if the font parameters are valid */
   font = __glcVerifyFontParameters(inFont);
   if (!font)
-    return GLC_NONE;
+    return NULL;
 
   ctx = GLC_GET_CURRENT_CONTEXT();
 
@@ -733,10 +746,15 @@ const GLCchar* APIENTRY glcGetFontListc(GLint inFont, GLCenum inAttrib,
   case GLC_FACE_LIST:
     return glcGetMasterListc(font->parentMasterID, inAttrib, inIndex);
   case GLC_CHAR_LIST:
-    return __glcCharMapGetCharNameByIndex(font->charMap, inIndex, ctx);
+    name = __glcCharMapGetCharNameByIndex(font->charMap, inIndex);
+    if (!name)
+      return NULL;
+
+    return __glcConvertFromUtf8ToBuffer(ctx, name,
+					ctx->stringState.stringType);
   default:
     __glcRaiseError(GLC_PARAMETER_ERROR);
-    return GLC_NONE;
+    return NULL;
   }
 }
 
@@ -767,22 +785,27 @@ const GLCchar* APIENTRY glcGetFontMap(GLint inFont, GLint inCode)
   __GLCfont *font = NULL;
   __GLCcontext *ctx = NULL;
   GLint code = 0;
+  const GLCchar8* name = NULL;
 
   GLC_INIT_THREAD();
 
   /* Check if the font parameters are valid */
   font = __glcVerifyFontParameters(inFont);
   if (!font)
-    return GLC_NONE;
+    return NULL;
 
   ctx = GLC_GET_CURRENT_CONTEXT();
 
   /* Get the character code converted to the UCS-4 format */
   code = __glcConvertGLintToUcs4(ctx, inCode);
   if (code < 0)
-    return GLC_NONE;
+    return NULL;
 
-  return __glcCharMapGetCharName(font->charMap, code, ctx);
+  name = __glcCharMapGetCharName(font->charMap, code);
+  if (!name)
+    return NULL;
+
+  return __glcConvertFromUtf8ToBuffer(ctx, name, ctx->stringState.stringType);
 }
 
 
