@@ -1,6 +1,6 @@
 /* QuesoGLC
  * A free implementation of the OpenGL Character Renderer (GLC)
- * Copyright (c) 2002, 2004-2008, Bertrand Coconnier
+ * Copyright (c) 2002, 2004-2009, Bertrand Coconnier
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU Lesser General Public
@@ -16,7 +16,7 @@
  *  License along with this library; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
-/* $Id: global.c 710 2008-01-21 20:14:17Z bcoconni $ */
+/* $Id: global.c 873 2009-01-18 17:51:17Z bcoconni $ */
 
 /** \file
  *  defines the so-called "Global commands" described in chapter 3.4 of the
@@ -60,16 +60,27 @@
 #include "internal.h"
 #include <stdlib.h>
 
+#ifdef __GNUC__
+__attribute__((constructor)) void init(void);
+__attribute__((destructor)) void fini(void);
+#else
+void _init(void);
+void _fini(void);
+#endif
+
+
 
 
 /* Since the common area can be accessed by any thread, this function should
  * be called before any access (read or write) to the common area. Otherwise
- * race conditions can occur.
+ * race conditions can occur. This function must also be used whenever we call
+ * a function which is not reentrant (it is the case for some Fontconfig
+ * entries).
  * __glcLock/__glcUnlock can be nested : they keep track of the number of
  * time they have been called and the mutex will be released as soon as
  * __glcUnlock() will be called as many time as __glcLock() was.
  */
-static void __glcLock(void)
+void __glcLock(void)
 {
   __GLCthreadArea *area = NULL;
 
@@ -92,7 +103,7 @@ static void __glcLock(void)
  * common area.
  * See also the note on nested calls in __glcLock's description.
  */
-static void __glcUnlock(void)
+void __glcUnlock(void)
 {
   __GLCthreadArea *area = NULL;
 
@@ -154,6 +165,10 @@ void _fini(void)
     node = next;
   }
 
+#if FC_MINOR > 2
+  FcFini();
+#endif
+
   __glcUnlock();
 #ifdef __WIN32__
   DeleteCriticalSection(&__glcCommonArea.section);
@@ -169,10 +184,6 @@ void _fini(void)
 
   pthread_key_delete(__glcCommonArea.threadKey);
 #endif
-
-#if FC_MINOR > 2
-  FcFini();
-#endif
 }
 
 
@@ -183,19 +194,19 @@ void _fini(void)
  */
 static void* __glcAllocFunc(FT_Memory GLC_UNUSED_ARG(inMemory), long inSize)
 {
-  return __glcMalloc(inSize);
+  return malloc(inSize);
 }
 
 static void __glcFreeFunc(FT_Memory GLC_UNUSED_ARG(inMemory), void *inBlock)
 {
-  __glcFree(inBlock);
+  free(inBlock);
 }
 
 static void* __glcReallocFunc(FT_Memory GLC_UNUSED_ARG(inMemory),
                               long GLC_UNUSED_ARG(inCurSize),
                               long inNewSize, void* inBlock)
 {
-  return __glcRealloc(inBlock, inNewSize);
+  return realloc(inBlock, inNewSize);
 }
 
 
