@@ -1,6 +1,6 @@
 /* QuesoGLC
  * A free implementation of the OpenGL Character Renderer (GLC)
- * Copyright (c) 2002, 2004-2007, Bertrand Coconnier
+ * Copyright (c) 2002, 2004-2008, Bertrand Coconnier
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU Lesser General Public
@@ -16,7 +16,7 @@
  *  License along with this library; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
-/* $Id: master.c,v 1.71 2007/10/28 15:23:38 bcoconni Exp $ */
+/* $Id: master.c 807 2008-06-07 18:33:28Z bcoconni $ */
 
 /** \file
  * defines the so-called "Master commands" described in chapter 3.6 of the GLC
@@ -49,8 +49,7 @@
  *  or equal to the value of the variable \b GLC_MASTER_COUNT.
  */
 
-#include "internal.h"
-#ifdef __WIN32__
+#if defined(__WIN32__) || defined(_MSC_VER)
 #include <io.h>
 #else
 #include <unistd.h>
@@ -58,6 +57,8 @@
 #include <sys/types.h>
 #include <sys/stat.h>
 
+#include "internal.h"
+
 
 
 /* Most master commands need to check that :
@@ -131,7 +132,8 @@ const GLCchar* APIENTRY glcGetMasterListc(GLint inMaster, GLCenum inAttrib,
   __GLCcontext *ctx = NULL;
   __GLCmaster *master = NULL;
   __GLCcharMap *charMap = NULL;
-  GLCchar8* string = NULL;
+  const GLCchar8* string = NULL;
+  GLCchar8* faceName = NULL;
   GLCchar* element = NULL;
 
   GLC_INIT_THREAD();
@@ -168,12 +170,12 @@ const GLCchar* APIENTRY glcGetMasterListc(GLint inMaster, GLCenum inAttrib,
   /* return the requested attribute */
   switch(inAttrib) {
   case GLC_CHAR_LIST:
-    charMap = __glcMasterGetCharMap(master, ctx);
+    charMap = __glcCharMapCreate(master, ctx);
     if (!charMap) {
       __glcMasterDestroy(master);
       return GLC_NONE;
     }
-    string = __glcCharMapGetCharNameByIndex(charMap, inIndex, ctx);
+    string = __glcCharMapGetCharNameByIndex(charMap, inIndex);
     if (!string) {
       __glcMasterDestroy(master);
       __glcCharMapDestroy(charMap);
@@ -182,7 +184,8 @@ const GLCchar* APIENTRY glcGetMasterListc(GLint inMaster, GLCenum inAttrib,
     break;
   case GLC_FACE_LIST:
     /* Get the face name */
-    string = __glcMasterGetFaceName(master, ctx, inIndex);
+    faceName = __glcMasterGetFaceName(master, ctx, inIndex);
+    string = (const GLCchar8*)faceName;
     break;
   default:
     __glcRaiseError(GLC_PARAMETER_ERROR);
@@ -197,7 +200,7 @@ const GLCchar* APIENTRY glcGetMasterListc(GLint inMaster, GLCenum inAttrib,
   if (charMap)
     __glcCharMapDestroy(charMap);
   else
-      free(string);
+    free(faceName);
 
   return element;
 }
@@ -243,26 +246,31 @@ const GLCchar* APIENTRY glcGetMasterMap(GLint inMaster, GLint inCode)
     __GLCcharMap* charMap = NULL;
     GLCchar* result = NULL;
     GLint code = 0;
+    const GLCchar8* name = NULL;
 
-    charMap = __glcMasterGetCharMap(master, ctx);
-    if (!charMap) {
-      __glcRaiseError(GLC_RESOURCE_ERROR);
-      __glcMasterDestroy(master);
-      return GLC_NONE;
-    }
+    charMap = __glcCharMapCreate(master, ctx);
+    __glcMasterDestroy(master);
+    if (!charMap)
+      return NULL;
 
     /* Get the character code converted to the UCS-4 format */
     code = __glcConvertGLintToUcs4(ctx, inCode);
-    if (code < 0)
-      return GLC_NONE;
+    if (code < 0) {
+      __glcCharMapDestroy(charMap);
+      return NULL;
+    }
 
-    result = __glcCharMapGetCharName(charMap, code, ctx);
-    __glcMasterDestroy(master);
+    name = __glcCharMapGetCharName(charMap, code);
     __glcCharMapDestroy(charMap);
+    if (!name)
+      return NULL;
+
+    result = __glcConvertFromUtf8ToBuffer(ctx, name,
+					  ctx->stringState.stringType);
     return result;
   }
   else
-    return GLC_NONE;
+    return NULL;
 }
 
 
@@ -304,7 +312,7 @@ const GLCchar* APIENTRY glcGetMasterMap(GLint inMaster, GLint inCode)
 const GLCchar* APIENTRY glcGetMasterc(GLint inMaster, GLCenum inAttrib)
 {
   __GLCcontext *ctx = NULL;
-  GLCchar *buffer = NULL;
+  const GLCchar *buffer = NULL;
   __GLCmaster* master = NULL;
 
   GLC_INIT_THREAD();
@@ -331,6 +339,7 @@ const GLCchar* APIENTRY glcGetMasterc(GLint inMaster, GLCenum inAttrib)
 
   ctx = GLC_GET_CURRENT_CONTEXT();
   buffer = __glcMasterGetInfo(master, ctx, inAttrib);
+
   __glcMasterDestroy(master);
 
   return buffer;
@@ -416,7 +425,7 @@ GLint APIENTRY glcGetMasteri(GLint inMaster, GLCenum inAttrib)
   }
 
   if (inAttrib != GLC_FACE_COUNT) {
-    charMap = __glcMasterGetCharMap(master, ctx);
+    charMap = __glcCharMapCreate(master, ctx);
     if (!charMap) {
       __glcMasterDestroy(master);
       return GLC_NONE;
@@ -465,14 +474,14 @@ static void __glcAddCatalog(const GLCchar* inCatalog, GLboolean inAppend)
   #ifdef __WIN32__
   if (_access((const char*)inCatalog, 0)) {
   #else
-  if (access((char *)inCatalog, R_OK) < 0) {
+  if (access((const char *)inCatalog, R_OK) < 0) {
   #endif
     /* May be something more explicit should be done */
     __glcRaiseError(GLC_PARAMETER_ERROR);
 	return;
   }
   /* Check that 'inCatalog' is a directory */
-  if (stat((char *)inCatalog, &dirStat) < 0) {
+  if (stat((const char *)inCatalog, &dirStat) < 0) {
     __glcRaiseError(GLC_PARAMETER_ERROR);
 	return;
   }
