@@ -1,6 +1,6 @@
 /* QuesoGLC
  * A free implementation of the OpenGL Character Renderer (GLC)
- * Copyright (c) 2002, 2004-2008, Bertrand Coconnier
+ * Copyright (c) 2002, 2004-2009, Bertrand Coconnier
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU Lesser General Public
@@ -16,7 +16,7 @@
  *  License along with this library; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
-/* $Id: misc.c 712 2008-01-26 19:46:49Z bcoconni $ */
+/* $Id: misc.c 887 2009-03-24 01:03:11Z bcoconni $ */
 
 /** \file
  * defines miscellaneous utility routines used throughout QuesoGLC.
@@ -27,45 +27,75 @@
 #include "internal.h"
 
 
+#ifdef DEBUGMODE
+GLCAPI GLuint __glcMemAllocCount = 0;
+GLCAPI GLuint __glcMemAllocTrigger = 0;
+GLCAPI GLboolean __glcMemAllocFailOnce = GL_TRUE;
+#endif
 
 /* QuesoGLC own allocation and memory management routines */
 void* __glcMalloc(size_t size)
 {
+#ifdef DEBUGMODE
+  __glcMemAllocCount++;
+
+  if (__glcMemAllocFailOnce) {
+    if (__glcMemAllocCount == __glcMemAllocTrigger)
+      return NULL;
+  }
+  else if (__glcMemAllocCount >= __glcMemAllocTrigger)
+    return NULL;
+#endif
   return malloc(size);
 }
 
 void __glcFree(void *ptr)
 {
+  /* Not all implementations of free() accept NULL. Moreover this allows to
+   * detect useless calls.
+   */
+  assert(ptr);
+
   free(ptr);
 }
 
 void* __glcRealloc(void *ptr, size_t size)
 {
+#ifdef DEBUGMODE
+  __glcMemAllocCount++;
+
+  if (__glcMemAllocFailOnce) {
+    if (__glcMemAllocCount == __glcMemAllocTrigger)
+      return NULL;
+  }
+  else if (__glcMemAllocCount >= __glcMemAllocTrigger)
+    return NULL;
+#endif
   return realloc(ptr, size);
 }
 
 
 
 /* Find a token in a list of tokens separated by 'separator' */
-GLCchar* __glcFindIndexList(const GLCchar* inString, GLuint inIndex,
-			    const GLCchar* inSeparator)
+GLCchar8* __glcFindIndexList(GLCchar8* inString, GLuint inIndex,
+			     const GLCchar8* inSeparator)
 {
-    GLuint occurence = 0;
-    char *s = (char *)inString;
-    char *sep = (char *)inSeparator;
+  GLuint occurence = 0;
+  GLCchar8* s = inString;
+  const GLCchar8* sep = inSeparator;
 
-    if (!inIndex)
-	return (GLCchar *)inString;
+  if (!inIndex)
+    return s;
 
-    for (; *s != '\0'; s++) {
-        if (*s == *sep) {
-            occurence++;
-            if (occurence == inIndex)
-                break;
-        }
+  for (; *s != '\0'; s++) {
+    if (*s == *sep) {
+      occurence++;
+      if (occurence == inIndex)
+	break;
     }
+  }
 
-    return (GLCchar *) s;
+  return (GLCchar8 *) s;
 }
 
 
