@@ -1,6 +1,6 @@
 /* QuesoGLC
  * A free implementation of the OpenGL Character Renderer (GLC)
- * Copyright (c) 2002, 2004-2007, Bertrand Coconnier
+ * Copyright (c) 2002, 2004-2009, Bertrand Coconnier
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU Lesser General Public
@@ -16,7 +16,7 @@
  *  License along with this library; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
-/* $Id: oarray.c 607 2007-03-11 12:45:43Z bcoconni $ */
+/* $Id: oarray.c 877 2009-02-14 14:23:53Z bcoconni $ */
 
 /** \file
  * defines the object __GLCarray which is an array which size can grow as some
@@ -189,3 +189,30 @@ char* __glcArrayInsertCell(__GLCarray* This, int inRank, int inCells)
 
   return newCell;
 }
+
+
+
+/* Duplicate an array */
+__GLCarray* __glcArrayDuplicate(__GLCarray* This)
+{
+  __GLCarray* duplicate = NULL;
+
+  duplicate = (__GLCarray*)__glcMalloc(sizeof(__GLCarray));
+  if (!duplicate) {
+    __glcRaiseError(GLC_RESOURCE_ERROR);
+    return NULL;
+  }
+
+  memcpy(duplicate, This, sizeof(__GLCarray));
+
+  duplicate->data = (char*)__glcMalloc(This->allocated * This->elementSize);
+  if (!duplicate->data) {
+    __glcFree(duplicate);
+    __glcRaiseError(GLC_RESOURCE_ERROR);
+    return NULL;
+  }
+
+  memcpy(duplicate->data, This->data, This->allocated * This->elementSize);
+
+  return duplicate;
+}
