@@ -16,7 +16,7 @@
  *  License along with this library; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
-/* $Id: ocharmap.h 681 2007-12-22 13:15:55Z bcoconni $ */
+/* $Id: ocharmap.h 800 2008-06-04 21:47:50Z bcoconni $ */
 
 /** \file
  * header of the object __GLCcharMap which manage the charmaps of both the fonts
@@ -48,12 +48,11 @@ void __glcCharMapDestroy(__GLCcharMap* This);
 void __glcCharMapAddChar(__GLCcharMap* This, GLint inCode,
 			 __GLCglyph* inGlyph);
 void __glcCharMapRemoveChar(__GLCcharMap* This, GLint inCode);
-GLCchar* __glcCharMapGetCharName(__GLCcharMap* This, GLint inCode,
-				 __GLCcontext* inContext);
+const GLCchar8* __glcCharMapGetCharName(__GLCcharMap* This, GLint inCode);
 __GLCglyph* __glcCharMapGetGlyph(__GLCcharMap* This, GLint inCode);
 GLboolean __glcCharMapHasChar(__GLCcharMap* This, GLint inCode);
-GLCchar* __glcCharMapGetCharNameByIndex(__GLCcharMap* This, GLint inIndex,
-					__GLCcontext* inContext);
+const GLCchar8* __glcCharMapGetCharNameByIndex(__GLCcharMap* This,
+					       GLint inIndex);
 GLint __glcCharMapGetCount(__GLCcharMap* This);
 GLint __glcCharMapGetMaxMappedCode(__GLCcharMap* This);
 GLint __glcCharMapGetMinMappedCode(__GLCcharMap* This);
