@@ -1,6 +1,6 @@
 /* QuesoGLC
  * A free implementation of the OpenGL Character Renderer (GLC)
- * Copyright (c) 2002, 2004-2008, Bertrand Coconnier
+ * Copyright (c) 2002, 2004-2009, Bertrand Coconnier
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU Lesser General Public
@@ -16,7 +16,7 @@
  *  License along with this library; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
-/* $Id: ofacedesc.c 710 2008-01-21 20:14:17Z bcoconni $ */
+/* $Id: ofacedesc.c 870 2009-01-18 12:01:27Z bcoconni $ */
 
 /** \file
  * defines the object __GLCfaceDescriptor that contains the description of a
@@ -81,7 +81,6 @@ __GLCfaceDescriptor* __glcFaceDescCreate(__GLCmaster* inMaster,
   FcPatternDestroy(pattern);
   if (!fontSet) {
     __glcRaiseError(GLC_RESOURCE_ERROR);
-    FcPatternDestroy(pattern);
     return NULL;
   }
 
@@ -274,7 +273,7 @@ FT_Error __glcFileOpen(FTC_FaceID inFile, FT_Library inLibrary,
 {
   __GLCfaceDescriptor* file = (__GLCfaceDescriptor*)inFile;
   GLCchar8 *fileName = NULL;
-  int index = 0;
+  int fileIndex = 0;
   FcResult result = FcResultMatch;
   FT_Error error;
 
@@ -282,10 +281,10 @@ FT_Error __glcFileOpen(FTC_FaceID inFile, FT_Library inLibrary,
   result = FcPatternGetString(file->pattern, FC_FILE, 0, &fileName);
   assert(result != FcResultTypeMismatch);
   /* get the index of the font in font file */
-  result = FcPatternGetInteger(file->pattern, FC_INDEX, 0, &index);
+  result = FcPatternGetInteger(file->pattern, FC_INDEX, 0, &fileIndex);
   assert(result != FcResultTypeMismatch);
 
-  error = FT_New_Face(inLibrary, (const char*)fileName, index, outFace);
+  error = FT_New_Face(inLibrary, (const char*)fileName, fileIndex, outFace);
 
   if (error) {
     __glcRaiseError(GLC_RESOURCE_ERROR);
@@ -369,12 +368,14 @@ GLboolean __glcFaceDescPrepareGlyph(__GLCfaceDescriptor* This,
   FTC_ScalerRec scaler;
 # endif
   FT_Size size = NULL;
+#else
+  FT_Error error;
 #endif
 
   /* If GLC_HINTING_QSO is enabled then perform hinting on the glyph while
    * loading it.
    */
-  if (!inContext->enableState.hinting)
+  if (!inContext->enableState.hinting && !inContext->enableState.glObjects)
     loadFlags |= FT_LOAD_NO_HINTING;
 
   /* Open the face */
@@ -383,12 +384,19 @@ GLboolean __glcFaceDescPrepareGlyph(__GLCfaceDescriptor* This,
      && (FREETYPE_MINOR < 1 \
          || (FREETYPE_MINOR == 1 && FREETYPE_PATCH < 8))
   font.face_id = (FTC_FaceID)This;
-  font.pix_width = (FT_UShort) (inScaleX *
+
+  if (inContext->enableState.glObjects) {
+    font.pix_width = (FT_UShort) inScaleX;
+    font.pix_height = (FT_UShort) inScaleY;
+  }
+  else {
+    font.pix_width = (FT_UShort) (inScaleX *
       (inContext->renderState.resolution < GLC_EPSILON ?
        72. : inContext->renderState.resolution) / 72.);
-  font.pix_height = (FT_UShort) (inScaleY *
+    font.pix_height = (FT_UShort) (inScaleY *
       (inContext->renderState.resolution < GLC_EPSILON ?
        72. : inContext->renderState.resolution) / 72.);
+  }
 
   if (FTC_Manager_Lookup_Size(inContext->cache, &font, &This->face, &size)) {
     __glcRaiseError(GLC_RESOURCE_ERROR);
@@ -399,10 +407,17 @@ GLboolean __glcFaceDescPrepareGlyph(__GLCfaceDescriptor* This,
   scaler.width = (FT_UInt)(inScaleX * 64.);
   scaler.height = (FT_UInt)(inScaleY * 64.);
   scaler.pixel = (FT_Int)0;
-  scaler.x_res = (FT_UInt)(inContext->renderState.resolution < GLC_EPSILON ?
-			   72 : inContext->renderState.resolution);
-  scaler.y_res = (FT_UInt)(inContext->renderState.resolution < GLC_EPSILON ?
-			   72 : inContext->renderState.resolution);
+
+  if (inContext->enableState.glObjects) {
+    scaler.x_res = 72;
+    scaler.y_res = 72;
+  }
+  else {
+    scaler.x_res = (FT_UInt)(inContext->renderState.resolution < GLC_EPSILON ?
+			     72 : inContext->renderState.resolution);
+    scaler.y_res = (FT_UInt)(inContext->renderState.resolution < GLC_EPSILON ?
+			     72 : inContext->renderState.resolution);
+  }
 
   if (FTC_Manager_LookupSize(inContext->cache, &scaler, &size)) {
     __glcRaiseError(GLC_RESOURCE_ERROR);
@@ -416,10 +431,17 @@ GLboolean __glcFaceDescPrepareGlyph(__GLCfaceDescriptor* This,
     return GL_FALSE;
 
   /* Select the size of the glyph */
-  if (FT_Set_Char_Size(This->face, (FT_F26Dot6)(inScaleX * 64.),
-		       (FT_F26Dot6)(inScaleY * 64.),
-		       (FT_UInt)inContext->renderState.resolution,
-		       (FT_UInt)inContext->renderState.resolution)) {
+  if (inContext->enableState.glObjects) {
+    error = FT_Set_Char_Size(This->face, (FT_F26Dot6)(inScaleX * 64.),
+			     (FT_F26Dot6)(inScaleY * 64.), 0, 0);
+  }
+  else {
+    error = FT_Set_Char_Size(This->face, (FT_F26Dot6)(inScaleX * 64.),
+			     (FT_F26Dot6)(inScaleY * 64.),
+			     (FT_UInt)inContext->renderState.resolution,
+			     (FT_UInt)inContext->renderState.resolution);
+  }
+  if (error) {
     __glcFaceDescClose(This);
     __glcRaiseError(GLC_RESOURCE_ERROR);
     return GL_FALSE;
@@ -526,9 +548,9 @@ GLfloat* __glcFaceDescGetAdvance(__GLCfaceDescriptor* This,
 /* Use FreeType to determine in which format the face is stored in its file :
  * Type1, TrueType, OpenType, ...
  */
-GLCchar8* __glcFaceDescGetFontFormat(__GLCfaceDescriptor* This,
-				     __GLCcontext* inContext,
-				     GLCenum inAttrib)
+const GLCchar8* __glcFaceDescGetFontFormat(__GLCfaceDescriptor* This,
+					   __GLCcontext* inContext,
+					   GLCenum inAttrib)
 {
   static GLCchar8 unknown[] = "Unknown";
 #ifndef FT_XFREE86_H
@@ -548,7 +570,7 @@ GLCchar8* __glcFaceDescGetFontFormat(__GLCfaceDescriptor* This,
   FT_WinFNT_HeaderRec aheader;
 #endif
   GLCuint count = 0;
-  GLCchar8* result = NULL;
+  const GLCchar8* result = NULL;
 
   /* Open the face */
 #ifdef GLC_FT_CACHE
@@ -567,7 +589,7 @@ GLCchar8* __glcFaceDescGetFontFormat(__GLCfaceDescriptor* This,
      * added to the public API. It can be safely used nonetheless as long as
      * the existence of FT_XFREE86_H is checked.
      */
-    result = (GLCchar8*)FT_Get_X11_Font_Format(face);
+    result = (const GLCchar8*)FT_Get_X11_Font_Format(face);
 #  ifndef GLC_FT_CACHE
     __glcFaceDescClose(This);
 #  endif /* GLC_FT_CACHE */
@@ -963,9 +985,9 @@ static int __glcNextPowerOf2(int value)
 
 /* Get the size of the bitmap in which the glyph will be rendered */
 GLboolean __glcFaceDescGetBitmapSize(__GLCfaceDescriptor* This, GLint* outWidth,
-                                     GLint *outHeight, GLint* outBoundingBox,
-                                     GLfloat inScaleX, GLfloat inScaleY,
-                                     int inFactor, __GLCcontext* inContext)
+                                     GLint *outHeight, GLfloat inScaleX,
+				     GLfloat inScaleY, GLint* outPixBoundingBox,
+				     int inFactor, __GLCcontext* inContext)
 {
   FT_Outline outline;
   FT_Matrix matrix;
@@ -978,28 +1000,12 @@ GLboolean __glcFaceDescGetBitmapSize(__GLCfaceDescriptor* This, GLint* outWidth,
 
   if (inContext->renderState.renderStyle == GLC_BITMAP) {
     GLfloat *transform = inContext->bitmapMatrix;
-    FT_Pos pitch = 0;
 
     /* compute glyph dimensions */
     matrix.xx = (FT_Fixed)(transform[0] * 65536. / inScaleX);
     matrix.xy = (FT_Fixed)(transform[2] * 65536. / inScaleY);
     matrix.yx = (FT_Fixed)(transform[1] * 65536. / inScaleX);
     matrix.yy = (FT_Fixed)(transform[3] * 65536. / inScaleY);
-
-    /* Get the bounding box of the glyph */
-    FT_Outline_Transform(&outline, &matrix);
-    FT_Outline_Get_CBox(&outline, &boundingBox);
-
-    /* Compute the sizes of the pixmap where the glyph will be drawn */
-    boundingBox.xMin = boundingBox.xMin & -64;	/* floor(xMin) */
-    boundingBox.yMin = boundingBox.yMin & -64;	/* floor(yMin) */
-    boundingBox.xMax = (boundingBox.xMax + 63) & -64;	/* ceiling(xMax) */
-    boundingBox.yMax = (boundingBox.yMax + 63) & -64;	/* ceiling(yMax) */
-
-    /* Calculate pitch to upper 8 byte boundary for 1 bit/pixel, i.e. ceil() */
-    pitch = (boundingBox.xMax - boundingBox.xMin + 511) >> 9;
-    *outWidth = pitch << 3;
-    *outHeight = (boundingBox.yMax - boundingBox.yMin) >> 6;
   }
   else {
     matrix.xy = 0;
@@ -1008,38 +1014,109 @@ GLboolean __glcFaceDescGetBitmapSize(__GLCfaceDescriptor* This, GLint* outWidth,
     if (inContext->enableState.glObjects) {
       matrix.xx = (FT_Fixed)((GLC_TEXTURE_SIZE << 16) / inScaleX);
       matrix.yy = (FT_Fixed)((GLC_TEXTURE_SIZE << 16) / inScaleY);
+    }
+    else {
+      matrix.xx = 65536 >> inFactor;
+      matrix.yy = 65536 >> inFactor;
+    }
+  }
 
-      FT_Outline_Transform(&outline, &matrix);
-      FT_Outline_Get_CBox(&outline, &boundingBox);
+  FT_Outline_Transform(&outline, &matrix);
+  FT_Outline_Get_CBox(&outline, &boundingBox);
+
+  if (inContext->renderState.renderStyle == GLC_BITMAP) {
+    FT_Pos pitch = 0;
+
+    outPixBoundingBox[0] = GLC_FLOOR_26_6(boundingBox.xMin);
+    outPixBoundingBox[1] = GLC_FLOOR_26_6(boundingBox.yMin);
+    outPixBoundingBox[2] = GLC_CEIL_26_6(boundingBox.xMax);
+    outPixBoundingBox[3] = GLC_CEIL_26_6(boundingBox.yMax);
+
+    /* Calculate pitch to upper 8 byte boundary for 1 bit/pixel, i.e. ceil() */
+    pitch = (outPixBoundingBox[2] - outPixBoundingBox[0] + 511) >> 9;
+
+    *outWidth = pitch << 3;
+    *outHeight = (outPixBoundingBox[3] - outPixBoundingBox[1]) >> 6;
+  }
+  else {
+    GLint width = 0;
+    GLint height = 0;
+
+    if (inContext->enableState.glObjects) {
+      GLfloat ratioX = 0.f;
+      GLfloat ratioY = 0.f;
+      GLfloat ratio = 0.f;
+
+      width = boundingBox.xMax - boundingBox.xMin;
+      height = boundingBox.yMax - boundingBox.yMin;
+
+      ratioX = width / (64.f * GLC_TEXTURE_SIZE);
+      ratioY = height / (64.f * GLC_TEXTURE_SIZE);
+
+      ratioX = (ratioX > 1.f) ? ratioX : 1.f;
+      ratioY = (ratioY > 1.f) ? ratioY : 1.f;
+      ratio = ((ratioX > ratioY) ? ratioX : ratioY);
 
       *outWidth = GLC_TEXTURE_SIZE;
       *outHeight = GLC_TEXTURE_SIZE;
 
       outline.flags |= FT_OUTLINE_HIGH_PRECISION;
+
+      if (ratio > 1.f) {
+	outPixBoundingBox[0] = boundingBox.xMin
+	  - ((GLint)((GLC_TEXTURE_SIZE << 5) - (width * 0.5f)) * ratio);
+	outPixBoundingBox[1] = boundingBox.yMin
+	  - ((GLint)((GLC_TEXTURE_SIZE << 5) - (height * 0.5f)) * ratio);
+	outPixBoundingBox[2] = outPixBoundingBox[0]
+	  + ((GLint)((GLC_TEXTURE_SIZE << 6) * ratio));
+	outPixBoundingBox[3] = outPixBoundingBox[1]
+	  + ((GLint)((GLC_TEXTURE_SIZE << 6) * ratio));
+
+	matrix.xx = (FT_Fixed)(65536.f / ratio);
+	matrix.yy = matrix.xx;
+
+	FT_Outline_Transform(&outline, &matrix);
+	FT_Outline_Get_CBox(&outline, &boundingBox);
+      }
+      else {
+	outPixBoundingBox[0] = boundingBox.xMin
+	  - ((GLC_TEXTURE_SIZE << 5) - (width >> 1));
+	outPixBoundingBox[1] = boundingBox.yMin
+	  - ((GLC_TEXTURE_SIZE << 5) - (height >> 1));
+	outPixBoundingBox[2] = outPixBoundingBox[0] + ((GLC_TEXTURE_SIZE - 1) << 6);
+	outPixBoundingBox[3] = outPixBoundingBox[1] + ((GLC_TEXTURE_SIZE - 1) << 6);
+      }
     }
     else {
-      matrix.xx = 65536 >> inFactor;
-      matrix.yy = 65536 >> inFactor;
+      width = (GLC_CEIL_26_6(boundingBox.xMax)
+	       - GLC_FLOOR_26_6(boundingBox.xMin)) >> 6;
+      height = (GLC_CEIL_26_6(boundingBox.yMax)
+		- GLC_FLOOR_26_6(boundingBox.yMin)) >> 6;
 
-      FT_Outline_Transform(&outline, &matrix);
-      FT_Outline_Get_CBox(&outline, &boundingBox);
+      *outWidth = __glcNextPowerOf2(width);
+      *outHeight = __glcNextPowerOf2(height);
 
-      *outWidth = __glcNextPowerOf2(
-              (boundingBox.xMax - boundingBox.xMin + 63) >> 6); /* ceil() */
-      *outHeight = __glcNextPowerOf2(
-              (boundingBox.yMax - boundingBox.yMin + 63) >> 6); /* ceil() */
+      *outWidth = (*outWidth > inContext->texture.width)
+	? *outWidth : inContext->texture.width;
+      *outHeight = (*outHeight > inContext->texture.heigth)
+	? *outHeight : inContext->texture.heigth;
+
+      if (*outWidth - width <= 1) *outWidth <<= 1;
+      if (*outHeight - height <= 1) *outHeight <<= 1;
 
       /* If the texture size is too small then give up */
       if ((*outWidth < 4) || (*outHeight < 4))
         return GL_FALSE;
+
+      outPixBoundingBox[0] = GLC_FLOOR_26_6(boundingBox.xMin)
+	- (((*outWidth - width) >> 1 ) << 6);
+      outPixBoundingBox[1] = GLC_FLOOR_26_6(boundingBox.yMin)
+	- (((*outHeight - height) >> 1) << 6);
+      outPixBoundingBox[2] = outPixBoundingBox[0] + ((*outWidth - 1) << 6);
+      outPixBoundingBox[3] = outPixBoundingBox[1] + ((*outHeight - 1) << 6);
     }
   }
 
-  outBoundingBox[0] = boundingBox.xMin;
-  outBoundingBox[1] = boundingBox.yMin;
-  outBoundingBox[2] = boundingBox.xMax;
-  outBoundingBox[3] = boundingBox.yMax;
-
   return GL_TRUE;
 }
 
@@ -1054,16 +1131,38 @@ GLboolean __glcFaceDescGetBitmap(__GLCfaceDescriptor* This, GLint inWidth,
   FT_BBox boundingBox;
   FT_Bitmap pixmap;
   FT_Matrix matrix;
+  FT_Pos dx = 0, dy = 0;
   FT_Face face = This->face;
+  FT_Pos width = 0, height = 0;
 
   assert(face);
 
   outline = face->glyph->outline;
   FT_Outline_Get_CBox(&outline, &boundingBox);
 
+  if ((inContext->renderState.renderStyle == GLC_BITMAP)
+      || (!inContext->enableState.glObjects)) {
+    dx = GLC_FLOOR_26_6(boundingBox.xMin);
+    dy = GLC_FLOOR_26_6(boundingBox.yMin);
+    if (inContext->renderState.renderStyle == GLC_TEXTURE) {
+      width = (GLC_CEIL_26_6(boundingBox.xMax) - dx) >> 6;
+      height = (GLC_CEIL_26_6(boundingBox.yMax) - dy) >> 6;
+      dx -= (((inWidth - width) >> 1) << 6);
+      dy -= (((inHeight - height) >> 1) << 6);
+    }
+  }
+  else {
+    dx = boundingBox.xMin;
+    dy = boundingBox.yMin;
+    width = boundingBox.xMax - dx;
+    height = boundingBox.yMax - dy;
+    dx -= (inWidth << 5) - (width >> 1);
+    dy -= (inHeight << 5) - (height >> 1);
+  }
+  
   pixmap.width = inWidth;
   pixmap.rows = inHeight;
-  pixmap.buffer = inBuffer;
+  pixmap.buffer = (unsigned char*)inBuffer;
 
   if (inContext->renderState.renderStyle == GLC_BITMAP) {
     /* Calculate pitch to upper 8 byte boundary for 1 bit/pixel, i.e. ceil() */
@@ -1087,7 +1186,7 @@ GLboolean __glcFaceDescGetBitmap(__GLCfaceDescriptor* This, GLint inWidth,
   /* translate the outline to match (0,0) with the glyph's lower left
    * corner
    */
-  FT_Outline_Translate(&outline, -boundingBox.xMin, -boundingBox.yMin);
+  FT_Outline_Translate(&outline, -dx, -dy);
 
   /* render the glyph */
   if (FT_Outline_Get_Bitmap(inContext->library, &outline, &pixmap)) {
@@ -1099,7 +1198,7 @@ GLboolean __glcFaceDescGetBitmap(__GLCfaceDescriptor* This, GLint inWidth,
     /* Prepare the outline for the next mipmap level :
      * a. Restore the outline initial position
      */
-    FT_Outline_Translate(&outline, boundingBox.xMin, boundingBox.yMin);
+    FT_Outline_Translate(&outline, dx, dy);
 
     /* b. Divide the character size by 2. */
     matrix.xx = 32768; /* 0.5 in FT_Fixed type */
@@ -1117,8 +1216,7 @@ GLboolean __glcFaceDescGetBitmap(__GLCfaceDescriptor* This, GLint inWidth,
 /* Chek if the outline of the glyph is empty (which means it is a spacing
  * character).
  */
-GLboolean __glcFaceDescOutlineEmpty(__GLCfaceDescriptor* This,
-                                    __GLCcontext* inContext)
+GLboolean __glcFaceDescOutlineEmpty(__GLCfaceDescriptor* This)
 {
   FT_Outline outline = This->face->glyph->outline;
 
