@@ -16,7 +16,7 @@
  *  License along with this library; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
-/* $Id: ofacedesc.h 708 2008-01-20 13:28:08Z bcoconni $ */
+/* $Id: ofacedesc.h 838 2008-10-18 21:35:03Z bcoconni $ */
 
 /** \file
  * header of the object __GLCfaceDescriptor that contains the description of a
@@ -67,9 +67,9 @@ GLfloat* __glcFaceDescGetAdvance(__GLCfaceDescriptor* This,
 				 GLCulong inGlyphIndex, GLfloat* outVec,
 				 GLfloat inScaleX, GLfloat inScaleY,
 				 __GLCcontext* inContext);
-GLCchar8* __glcFaceDescGetFontFormat(__GLCfaceDescriptor* This,
-				    __GLCcontext* inContext,
-				    GLCenum inAttrib);
+const GLCchar8* __glcFaceDescGetFontFormat(__GLCfaceDescriptor* This,
+					   __GLCcontext* inContext,
+					   GLCenum inAttrib);
 GLfloat* __glcFaceDescGetMaxMetric(__GLCfaceDescriptor* This, GLfloat* outVec,
 				   __GLCcontext* inContext);
 GLfloat* __glcFaceDescGetKerning(__GLCfaceDescriptor* This,
@@ -82,14 +82,13 @@ GLboolean __glcFaceDescOutlineDecompose(__GLCfaceDescriptor* This,
                                         __GLCrendererData* inData,
                                         __GLCcontext* inContext);
 GLboolean __glcFaceDescGetBitmapSize(__GLCfaceDescriptor* This, GLint* outWidth,
-                                     GLint *outHeight, GLint* outBoundingBox,
-                                     GLfloat inScaleX, GLfloat inScaleY,
-                                     int inFactor, __GLCcontext* inContext);
+                                     GLint *outHeight, GLfloat inScaleX,
+				     GLfloat inScaleY, GLint* outPixBoundingBox,
+				     int inFactor, __GLCcontext* inContext);
 GLboolean __glcFaceDescGetBitmap(__GLCfaceDescriptor* This, GLint inWidth,
                                  GLint inHeight, void* inBuffer,
                                  __GLCcontext* inContext);
-GLboolean __glcFaceDescOutlineEmpty(__GLCfaceDescriptor* This,
-                                    __GLCcontext* inContext);
+GLboolean __glcFaceDescOutlineEmpty(__GLCfaceDescriptor* This);
 __GLCcharMap* __glcFaceDescGetCharMap(__GLCfaceDescriptor* This,
 				      __GLCcontext* inContext);
 #endif /* __glc_ofacedesc_h */
