@@ -16,7 +16,7 @@
  *  License along with this library; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
-/* $Id: ofont.c 710 2008-01-21 20:14:17Z bcoconni $ */
+/* $Id: ofont.c 848 2008-11-02 12:56:00Z bcoconni $ */
 
 /* Defines the methods of an object that is intended to managed fonts */
 
@@ -24,6 +24,7 @@
  * defines the object __GLCfont which manage the fonts
  */
 
+#include <math.h>
 #include "internal.h"
 
 
@@ -145,6 +146,21 @@ GLfloat* __glcFontGetBoundingBox(__GLCfont *This, GLint inCode,
 				   inScaleX, inScaleY, inContext))
     return NULL;
 
+  /* Special case for glyphes which have no bounding box (i.e. spaces) */
+  if ((fabs(outVec[0] - outVec[2]) < GLC_EPSILON)
+      || (fabs(outVec[1] - outVec[3]) < GLC_EPSILON)) {
+    GLfloat advance[2] = {0.f, 0.f};
+
+    if (__glcFontGetAdvance(This, inCode, advance, inContext, inScaleX,
+			    inScaleY)) {
+      if (fabs(outVec[0] - outVec[2]) < GLC_EPSILON)
+	outVec[2] += advance[0];
+
+      if (fabs(outVec[1] - outVec[3]) < GLC_EPSILON)
+	outVec[3] += advance[1];
+    }
+  }
+
   /* Copy the result to outVec and return */
   if (inContext->enableState.glObjects) {
     memcpy(glyph->boundingBox, outVec, 4 * sizeof(GLfloat));
@@ -282,24 +298,6 @@ GLboolean __glcFontFace(__GLCfont* This, const GLCchar8* inFace,
 
 
 
-#ifndef GLC_FT_CACHE
-/* Open the font file */
-inline void* __glcFontOpen(__GLCfont* This, __GLCcontext* inContext)
-{
-  return __glcFaceDescOpen(This->faceDesc, inContext);
-}
-
-
-
-/* Close the font file */
-inline void __glcFontClose(__GLCfont* This)
-{
-   __glcFaceDescClose(This->faceDesc);
-}
-#endif
-
-
-
 /* Load a glyph of the current font face and stores the corresponding data in
  * the corresponding face. The size of the glyph is given by inScaleX and
  * inScaleY. 'inGlyphIndex' contains the index of the glyph in the font file.
@@ -317,58 +315,3 @@ GLboolean __glcFontPrepareGlyph(__GLCfont* This, __GLCcontext* inContext,
 
   return result;
 }
-
-
-
-/* Get the size of the bitmap in which the glyph will be rendered */
-inline GLboolean __glcFontGetBitmapSize(__GLCfont* This, GLint* outWidth,
-					GLint *outHeight, GLint* outBoundingBox,
-					GLfloat inScaleX, GLfloat inScaleY,
-					int inFactor, __GLCcontext* inContext)
-{
-  return __glcFaceDescGetBitmapSize(This->faceDesc, outWidth, outHeight,
-				    outBoundingBox, inScaleX, inScaleY,
-				    inFactor, inContext);
-}
-
-
-
-/* Get the maximum metrics of a face that is the bounding box that encloses
- * every glyph of the face, and the maximum advance of the face.
- */
-inline GLfloat* __glcFontGetMaxMetric(__GLCfont* This, GLfloat* outVec,
-				      __GLCcontext* inContext)
-{
-  return __glcFaceDescGetMaxMetric(This->faceDesc, outVec, inContext);
-}
-
-
-
-/* Decompose the outline of a glyph */
-inline GLboolean __glcFontOutlineDecompose(__GLCfont* This,
-					   __GLCrendererData* inData,
-					   __GLCcontext* inContext)
-{
-  return __glcFaceDescOutlineDecompose(This->faceDesc, inData, inContext);
-}
-
-
-
-/* Render the glyph in a bitmap */
-inline GLboolean __glcFontGetBitmap(__GLCfont* This, GLint inWidth,
-				    GLint inHeight, void* inBuffer,
-				    __GLCcontext* inContext)
-{
-  return __glcFaceDescGetBitmap(This->faceDesc, inWidth, inHeight, inBuffer,
-				inContext);
-}
-
-
-
-/* Chek if the outline of the glyph is empty (which means it is a spacing
- * character).
- */
-inline GLboolean __glcFontOutlineEmpty(__GLCfont* This, __GLCcontext* inContext)
-{
-  return __glcFaceDescOutlineEmpty(This->faceDesc, inContext);
-}
