@@ -16,7 +16,7 @@
  *  License along with this library; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
-/* $Id: oglyph.c 709 2008-01-21 20:04:13Z bcoconni $ */
+/* $Id: oglyph.c 849 2008-11-02 13:02:23Z bcoconni $ */
 
 /* Defines the methods of an object that is intended to managed glyphs */
 
@@ -96,7 +96,7 @@ void __glcGlyphDestroyTexture(__GLCglyph* This, __GLCcontext* inContext)
 void __glcGlyphDestroyGLObjects(__GLCglyph* This, __GLCcontext* inContext)
 {
   if (This->glObject[1]) {
-    __glcDeleteAtlasElement(This->textureObject, inContext);
+    __glcReleaseAtlasElement(This->textureObject, inContext);
     __glcGlyphDestroyTexture(This, inContext);
   }
 
