@@ -16,7 +16,7 @@
  *  License along with this library; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
-/* $Id: omaster.c 710 2008-01-21 20:14:17Z bcoconni $ */
+/* $Id: omaster.c 824 2008-08-09 19:09:36Z bcoconni $ */
 
 /** \file
  * defines the object __GLCmaster which manage the masters
@@ -169,7 +169,6 @@ GLCchar8* __glcMasterGetFaceName(__GLCmaster* This, __GLCcontext* inContext,
     int fixed = 0;
     FcChar8* foundry = NULL;
     FcBool outline = FcFalse;
-    FcResult result = FcResultMatch;
     FcBool equal = FcFalse;
 
     /* Check whether the glyphs are outlines */
@@ -330,58 +329,65 @@ GLint __glcMasterFaceCount(__GLCmaster* This, __GLCcontext* inContext)
  * accesses, informations such as the master format, full name or version are
  * read "just in time" i.e. only when the user requests them.
  */
-GLCchar8* __glcMasterGetInfo(__GLCmaster* This, __GLCcontext* inContext,
-			     GLCenum inAttrib)
+const GLCchar8* __glcMasterGetInfo(__GLCmaster* This, __GLCcontext* inContext,
+				   GLCenum inAttrib)
 {
   __GLCfaceDescriptor* faceDesc = NULL;
   FcResult result = FcResultMatch;
-  GLCchar8* string = NULL;
-  GLCchar *buffer = NULL;
+  GLCchar8 *string = NULL;
+  const GLCchar8* info = NULL;
+  const GLCchar8 *buffer = NULL;
 
   /* Get the Unicode string which corresponds to the requested attribute */
   switch(inAttrib) {
   case GLC_FAMILY:
     result = FcPatternGetString(This->pattern, FC_FAMILY, 0, &string);
     assert(result != FcResultTypeMismatch);
-    break;
+    return string;
   case GLC_VENDOR:
     result = FcPatternGetString(This->pattern, FC_FOUNDRY, 0, &string);
     assert(result != FcResultTypeMismatch);
-    break;
+    return string;
   case GLC_VERSION:
   case GLC_FULL_NAME_SGI:
   case GLC_MASTER_FORMAT:
     faceDesc = __glcFaceDescCreate(This, NULL, inContext, 0);
 
-#ifdef GLC_FT_CACHE
-    if (!faceDesc) {
-#else
-    if (!faceDesc || !__glcFaceDescOpen(faceDesc, inContext)) {
-#endif
+    if (!faceDesc)
+      return NULL;
+
+#ifndef GLC_FT_CACHE
+    if (!__glcFaceDescOpen(faceDesc, inContext)) {
       __glcFaceDescDestroy(faceDesc, inContext);
       return NULL;
     }
-
-    string = __glcFaceDescGetFontFormat(faceDesc, inContext, inAttrib);
-    break;
-  }
-
-  if (string) {
-    /* Convert the string and store it in the context buffer */
-    buffer = __glcConvertFromUtf8ToBuffer(inContext, string,
-					  inContext->stringState.stringType);
-  }
-  else
-    __glcRaiseError(GLC_RESOURCE_ERROR);
-
-  if (faceDesc) {
-#ifndef GLC_FT_CACHE
-    __glcFaceDescClose(faceDesc);
 #endif
-    __glcFaceDescDestroy(faceDesc, inContext);
+
+    info = __glcFaceDescGetFontFormat(faceDesc, inContext, inAttrib);
+    if (info) {
+      /* Convert the string and store it in the context buffer */
+      buffer = (const GLCchar8*)__glcConvertFromUtf8ToBuffer(inContext, info,
+					    inContext->stringState.stringType);
+    }
+    else
+      __glcRaiseError(GLC_RESOURCE_ERROR);
+
+    if (faceDesc) {
+#ifndef GLC_FT_CACHE
+      __glcFaceDescClose(faceDesc);
+#endif
+      __glcFaceDescDestroy(faceDesc, inContext);
+    }
+
+    return buffer;
   }
 
-  return buffer;
+
+  /* The program is not supposed to reach the end of the function.
+   * The following return is there to prevent the compiler to issue
+   * a warning about 'control reaching the end of a non-void function'.
+   */
+    return (const GLCchar8*)"";
 }
 
 
