@@ -1,6 +1,6 @@
 /* QuesoGLC
  * A free implementation of the OpenGL Character Renderer (GLC)
- * Copyright (c) 2002, 2004-2008, Bertrand Coconnier
+ * Copyright (c) 2002, 2004-2009, Bertrand Coconnier
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU Lesser General Public
@@ -16,7 +16,7 @@
  *  License along with this library; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
-/* $Id: render.c 749 2008-03-08 20:18:29Z bcoconni $ */
+/* $Id: render.c 853 2009-01-10 14:02:37Z bcoconni $ */
 
 /** \file
  * defines the so-called "Rendering commands" described in chapter 3.9 of the
@@ -91,6 +91,7 @@
 #else
 #include <GL/glu.h>
 #endif
+#include <math.h>
 
 #include "texture.h"
 
@@ -104,11 +105,11 @@ static void __glcRenderCharBitmap(__GLCfont* inFont, __GLCcontext* inContext,
 {
   GLfloat *transform = inContext->bitmapMatrix;
   GLint pixWidth = 0, pixHeight = 0;
-  void* pixBuffer = NULL;
-  GLint boundingBox[4] = {0, 0, 0, 0};
+  GLubyte* pixBuffer = NULL;
+  GLint pixBoundingBox[4] = {0, 0, 0, 0};
 
-  __glcFontGetBitmapSize(inFont, &pixWidth, &pixHeight, boundingBox, scale_x,
-			 scale_y, 0, inContext);
+  __glcFontGetBitmapSize(inFont, &pixWidth, &pixHeight, scale_x, scale_y, 0,
+			 pixBoundingBox, inContext);
 
   pixBuffer = (GLubyte *)__glcMalloc(pixWidth * pixHeight);
   if (!pixBuffer) {
@@ -135,12 +136,12 @@ static void __glcRenderCharBitmap(__GLCfont* inFont, __GLCcontext* inContext,
 	     advance[1] * transform[2] - advance[0] * transform[0],
 	     advance[1] * transform[3] - advance[0] * transform[1],
 	     NULL);
-    glBitmap(pixWidth, pixHeight, -boundingBox[0] >> 6,
-	     -boundingBox[1] >> 6, 0., 0., pixBuffer);
+    glBitmap(pixWidth, pixHeight, -pixBoundingBox[0] >> 6,
+	     -pixBoundingBox[1] >> 6, 0., 0., pixBuffer);
   }
   else
-    glBitmap(pixWidth, pixHeight, -boundingBox[0] >> 6,
-	     -boundingBox[1] >> 6,
+    glBitmap(pixWidth, pixHeight, -pixBoundingBox[0] >> 6,
+	     -pixBoundingBox[1] >> 6,
 	     advance[0] * transform[0] + advance[1] * transform[2],
 	     advance[0] * transform[1] + advance[1] * transform[3],
 	     pixBuffer);
@@ -171,7 +172,7 @@ static void* __glcRenderChar(GLint inCode, GLint inPrevCode, GLboolean inIsRTL,
 
   __glcGetScale(inContext, transformMatrix, &scale_x, &scale_y);
 
-  if ((scale_x == 0.f) || (scale_y == 0.f))
+  if ((fabs(scale_x) < GLC_EPSILON) || (fabs(scale_y) < GLC_EPSILON))
     return NULL;
 
 #ifndef GLC_FT_CACHE
@@ -231,7 +232,7 @@ static void* __glcRenderChar(GLint inCode, GLint inPrevCode, GLboolean inIsRTL,
     /* If the outline contains no point then the glyph represents a space
      * character and there is no need to continue the process of rendering.
      */
-    if (!__glcFontOutlineEmpty(inFont, inContext)) {
+    if (!__glcFontOutlineEmpty(inFont)) {
       /* Update the advance and return */
       if (!inIsRTL)
         glTranslatef(advance[0], advance[1], 0.f);
@@ -278,7 +279,7 @@ static void* __glcRenderChar(GLint inCode, GLint inPrevCode, GLboolean inIsRTL,
     if (!inContext->enableState.glObjects)
       glScalef(sx64, sy64, 1.);
     if (!inIsRTL)
-      glTranslatef(advance[0], advance[1], 0.);
+      glTranslatef(advance[0], advance[1], 0.f);
   }
 #ifndef GLC_FT_CACHE
   __glcFontClose(inFont);
@@ -292,7 +293,8 @@ static void* __glcRenderChar(GLint inCode, GLint inPrevCode, GLboolean inIsRTL,
  * glcRenderCountedString(). The string 'inString' must be sorted in visual
  * order and stored using UCS4 format.
  */
-static void __glcRenderCountedString(__GLCcontext* inContext, GLCchar* inString,
+static void __glcRenderCountedString(__GLCcontext* inContext,
+				     GLCchar32* inString,
 				     GLboolean inIsRightToLeft,GLint inCount)
 {
   GLint listIndex = 0;
@@ -376,6 +378,8 @@ static void __glcRenderCountedString(__GLCcontext* inContext, GLCchar* inString,
     int j = 0;
     GLuint GLObjectIndex = inContext->renderState.renderStyle - 0x101;
     FT_ListNode node = NULL;
+    float resolution = (inContext->renderState.resolution < GLC_EPSILON ?
+			72. : inContext->renderState.resolution) / 72.;
 
     if (inContext->renderState.renderStyle == GLC_TRIANGLE
 	&& inContext->enableState.extrude)
@@ -403,18 +407,26 @@ static void __glcRenderCountedString(__GLCcontext* inContext, GLCchar* inString,
  	    if (inContext->enableState.kerning) {
  	      if (prevCode.code && prevCode.font == font) {
  		GLfloat kerning[2];
- 		GLint leftCode = inIsRightToLeft ? *ptr : prevCode.code;
- 		GLint rightCode = inIsRightToLeft ? prevCode.code : *ptr;
+		GLint leftCode = inIsRightToLeft ? *ptr : prevCode.code;
+		GLint rightCode = inIsRightToLeft ? prevCode.code : *ptr;
 
  		if (__glcFontGetKerning(font, leftCode, rightCode, kerning,
  					inContext, GLC_POINT_SIZE,
  					GLC_POINT_SIZE)) {
- 		  if (inIsRightToLeft)
- 		    chars[length].advance[0] -= kerning[0];
- 		  else
- 		    chars[length].advance[0] += kerning[0];
+		  if (!length) {
+		    if (inIsRightToLeft)
+		      glTranslatef(-kerning[0], kerning[1], 0.f);
+		    else
+		      glTranslatef(kerning[0], kerning[1], 0.f);
+		  }
+		  else {
+		    if (inIsRightToLeft)
+		      chars[length - 1].advance[0] -= kerning[0];
+		    else
+		      chars[length - 1].advance[0] += kerning[0];
 
- 		  chars[length].advance[1] += kerning[1];
+		    chars[length - 1].advance[1] += kerning[1];
+		  }
  		}
  	      }
  	    }
@@ -434,11 +446,13 @@ static void __glcRenderCountedString(__GLCcontext* inContext, GLCchar* inString,
       }
 
       if(!node || (i == inCount-1)) {
+	glScalef(resolution, resolution, 1.f);
+
 	for (j = 0; j < length; j++) {
 	  if (inIsRightToLeft)
 	    glTranslatef(-chars[j].advance[0], chars[j].advance[1], 0.);
 	  if (chars[j].code != 32) {
-	    __GLCglyph* glyph = chars[j].glyph;
+	    glyph = chars[j].glyph;
 
 	    switch(inContext->renderState.renderStyle) {
 	    case GLC_TEXTURE:
@@ -475,6 +489,8 @@ static void __glcRenderCountedString(__GLCcontext* inContext, GLCchar* inString,
 	if (!node)
 	  __glcProcessChar(inContext, *ptr, &prevCode, inIsRightToLeft,
 			   __glcRenderChar, NULL);
+
+	glScalef(1./resolution, 1./resolution, 1.f);
 	length = 0;
       }
 
@@ -536,7 +552,7 @@ void APIENTRY glcRenderChar(GLint inCode)
   if (code < 32)
     return; /* Skip control characters and unknown characters */
 
-  __glcRenderCountedString(ctx, (GLCchar*)&code, GL_FALSE, 1);
+  __glcRenderCountedString(ctx, (GLCchar32*)&code, GL_FALSE, 1);
 }
 
 
@@ -733,15 +749,24 @@ void APIENTRY glcReplacementCode(GLint inCode)
  *
  *  The resolution is given in \e dpi (dots per inch). If \e inVal is zero, the
  *  resolution defaults to 72 dpi.
+ *
+ *  The command raises \b GLC_PARAMETER_ERROR if \e inVal is negative.
  *  \param inVal A floating point number to be used as resolution.
  *  \sa glcGeti() with argument GLC_RESOLUTION
  */
 void APIENTRY glcResolution(GLfloat inVal)
 {
   __GLCcontext *ctx = NULL;
+  FT_ListNode node = NULL;
 
   GLC_INIT_THREAD();
 
+  /* Negative resolutions are illegal */
+  if (inVal < 0) {
+    __glcRaiseError(GLC_PARAMETER_ERROR);
+    return;
+  }
+
   /* Check if the current thread owns a current state */
   ctx = GLC_GET_CURRENT_CONTEXT();
   if (!ctx) {
@@ -752,5 +777,18 @@ void APIENTRY glcResolution(GLfloat inVal)
   /* Stores the resolution */
   ctx->renderState.resolution = inVal;
 
-  return;
+  /* Force the measurement caches to be updated */
+  for (node = ctx->fontList.head; node; node = node->next) {
+    __GLCfont* font = (__GLCfont*)node->data;
+    __GLCfaceDescriptor* faceDesc = font->faceDesc;
+    FT_ListNode glyphNode = NULL;
+
+    for (glyphNode = faceDesc->glyphList.head; glyphNode;
+	 glyphNode = glyphNode->next) {
+      __GLCglyph* glyph = (__GLCglyph*)glyphNode->data;
+
+      glyph->advanceCached = GL_FALSE;
+      glyph->boundingBoxCached = GL_FALSE;
+    }
+  }
 }
