@@ -1,6 +1,6 @@
 /* QuesoGLC
  * A free implementation of the OpenGL Character Renderer (GLC)
- * Copyright (c) 2002, 2004-2008, Bertrand Coconnier
+ * Copyright (c) 2002, 2004-2009, Bertrand Coconnier
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU Lesser General Public
@@ -16,7 +16,7 @@
  *  License along with this library; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
-/* $Id: texture.c 748 2008-03-08 20:15:34Z bcoconni $ */
+/* $Id: texture.c 879 2009-02-22 16:49:24Z bcoconni $ */
 
 /** \file
  *  defines the routines used to render characters with textures.
@@ -34,20 +34,25 @@
 
 
 
-/* This function is called when a glyph is destroyed, so that it is removed from
- * the atlas list if relevant.
+/* This function is called when a glyph is destroyed, the atlas element is then
+ * released.
  */
-void __glcDeleteAtlasElement(__GLCatlasElement* This,
+void __glcReleaseAtlasElement(__GLCatlasElement* This,
 			     __GLCcontext* inContext)
 {
-  FT_List_Remove(&inContext->atlasList, (FT_ListNode)This);
-  inContext->atlasCount--;
-  free(This);
+  FT_ListNode node = (FT_ListNode)This;
+
+  /* Put the atlas element at the tail of the list so that its position is used
+   * as soon as possible.
+   */
+  FT_List_Remove(&inContext->atlasList, node);
+  FT_List_Add(&inContext->atlasList, node);
+  This->glyph = NULL; /* The glyph will be destroyed so clear the pointer */
 }
 
 
 
-/* This function get some room in the texture atlas for a new glyph 'inGlyph'.
+/* This function gets some room in the texture atlas for a new glyph 'inGlyph'.
  * Eventually it creates the texture atlas, if it does not exist yet.
  */
 static GLboolean __glcTextureAtlasGetPosition(__GLCcontext* inContext,
@@ -61,6 +66,7 @@ static GLboolean __glcTextureAtlasGetPosition(__GLCcontext* inContext,
     int i = 0;
     GLint format = 0;
     GLint level = 0;
+    void * buffer = NULL;
 
     /* Not all gfx card are able to use 1024x1024 textures (especially old ones
      * like 3dfx's). Moreover, the texture memory may be scarce when our texture
@@ -87,6 +93,13 @@ static GLboolean __glcTextureAtlasGetPosition(__GLCcontext* inContext,
       return GL_FALSE;
     }
 
+    buffer = __glcMalloc(size * size);
+    if (!buffer) {
+      __glcRaiseError(GLC_RESOURCE_ERROR);
+      return GL_FALSE;
+    }
+    memset(buffer, 0, size * size);
+
     /* Create the texture atlas structure. The texture is divided in small
      * square areas of GLC_TEXTURE_SIZE x GLC_TEXTURE_SIZE, each of which will
      * contain a different glyph.
@@ -101,7 +114,7 @@ static GLboolean __glcTextureAtlasGetPosition(__GLCcontext* inContext,
     inContext->atlasCount = 0;
     glBindTexture(GL_TEXTURE_2D, inContext->atlas.id);
     glTexImage2D(GL_TEXTURE_2D, 0, GL_ALPHA8, size,
-		 size, 0, GL_ALPHA, GL_UNSIGNED_BYTE, NULL);
+		 size, 0, GL_ALPHA, GL_UNSIGNED_BYTE, buffer);
 
     /* Create the mipmap structure of the texture atlas, no matter if GLC_MIPMAP
      * is enabled or not.
@@ -110,21 +123,62 @@ static GLboolean __glcTextureAtlasGetPosition(__GLCcontext* inContext,
       size >>= 1;
       level++;
       glTexImage2D(GL_TEXTURE_2D, level, GL_ALPHA8, size,
-		   size, 0, GL_ALPHA, GL_UNSIGNED_BYTE, NULL);
+		   size, 0, GL_ALPHA, GL_UNSIGNED_BYTE, buffer);
     }
 
     /* Use trilinear filtering if GLC_MIPMAP is enabled.
      * Otherwise use bilinear filtering.
      */
+    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER,
+		    GL_LINEAR_MIPMAP_LINEAR);
+    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
+    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP);
+    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP);
+
+    /* The intent of this code is to work around an ugly bug of the Intel GMA
+     * 965 (or X3000) drivers on Linux. On those crappy drivers a 2nd call to
+     * glTexSubImage2D() completely clears the texture removing by the way the
+     * first character stored in the texture...
+     * This workaround displays a dummy character in order to deceive the
+     * stupid drivers. Note that I tried to reduce the code to the minimum : it
+     * seems that if any line below is removed, the workaround no longer works
+     * around the f***ing bug.
+     */
+    size = GLC_TEXTURE_SIZE;
+    glTexSubImage2D(GL_TEXTURE_2D, 0, 0, 0, size, size, GL_ALPHA,
+		    GL_UNSIGNED_BYTE, buffer);
+    level = 0;
+    while (size > 2) {
+      size >>= 1;
+      level++;
+      glTexSubImage2D(GL_TEXTURE_2D, level, 0, 0, size, size, GL_ALPHA,
+		      GL_UNSIGNED_BYTE, buffer);
+    }
+
+    if (GLEW_VERSION_1_2 || GLEW_SGIS_texture_lod)
+      glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAX_LEVEL, level - 1);
+
+    glBegin(GL_QUADS);
+    glNormal3f(0.f, 0.f, 1.f);
+    glTexCoord2f(0.f, 0.f);
+    glVertex2f(0.f, 0.f);
+    glTexCoord2f(0.f, 1.f);
+    glVertex2f(0.f, .5f);
+    glTexCoord2f(1.f, 1.f);
+    glVertex2f(.5f, .5f);
+    glTexCoord2f(1.f, 0.f);
+    glVertex2f(.5f, 0.f);
+    glEnd();
+    /* End of the workaround for the crappy open source drivers for Intel chips
+     */
+    __glcFree(buffer);
+
     if (inContext->enableState.mipmap)
       glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER,
 		      GL_LINEAR_MIPMAP_LINEAR);
     else
       glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER,
 		      GL_LINEAR);
-    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
-    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP);
-    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP);
   }
 
   /* At this stage, we want to get a free area in the texture atlas in order to
@@ -137,8 +191,11 @@ static GLboolean __glcTextureAtlasGetPosition(__GLCcontext* inContext,
      */
     atlasNode = (__GLCatlasElement*)inContext->atlasList.tail;
     assert(atlasNode);
-    /* Release the texture area of the glyph */
-    __glcGlyphDestroyTexture(atlasNode->glyph, inContext);
+
+    if (atlasNode->glyph) {
+      /* Release the texture area of the glyph */
+      __glcGlyphDestroyTexture(atlasNode->glyph, inContext);
+    }
     /* Put the texture area at the head of the list otherwise we will use the
      * same texture element over and over again each time that we need to
      * release a texture area.
@@ -180,7 +237,6 @@ static GLboolean __glcTextureAtlasGetPosition(__GLCcontext* inContext,
     glBindBufferARB(GL_ARRAY_BUFFER_ARB, inContext->atlas.bufferObjectID);
   }
 
-
   return GL_TRUE;
 }
 
@@ -282,14 +338,13 @@ void __glcRenderCharTexture(__GLCfont* inFont, __GLCcontext* inContext,
 			    GLfloat scale_x, GLfloat scale_y,
 			    __GLCglyph* inGlyph)
 {
-  GLfloat width = 0, heigth = 0;
   GLint level = 0;
-  GLint posX = 0, posY = 0;
+  GLint texX = 0, texY = 0;
   GLint pixWidth = 0, pixHeight = 0;
   void* pixBuffer = NULL;
-  GLint boundingBox[4] = {0, 0, 0, 0};
+  GLint pixBoundingBox[4] = {0, 0, 0, 0};
   int minSize = (GLEW_VERSION_1_2 || GLEW_SGIS_texture_lod) ? 2 : 1;
-  GLfloat texWidth = 0, texHeigth = 0;
+  GLfloat texWidth = 0.f, texHeight = 0.f;
 
   if (inContext->enableState.glObjects) {
     __GLCatlasElement* atlasNode = NULL;
@@ -300,14 +355,14 @@ void __glcRenderCharTexture(__GLCfont* inFont, __GLCcontext* inContext,
     /* Compute the size of the pixmap where the glyph will be rendered */
     atlasNode = inGlyph->textureObject;
 
-    __glcFaceDescGetBitmapSize(inFont->faceDesc, &pixWidth, &pixHeight,
-                               boundingBox, scale_x, scale_y, 0, inContext);
+    __glcFaceDescGetBitmapSize(inFont->faceDesc, &pixWidth, &pixHeight, scale_x,
+			       scale_y, pixBoundingBox, 0, inContext);
 
     texWidth = inContext->atlas.width;
-    texHeigth = inContext->atlas.heigth;
-    posY = (atlasNode->position / inContext->atlasWidth);
-    posX = (atlasNode->position - posY*inContext->atlasWidth)*GLC_TEXTURE_SIZE;
-    posY *= GLC_TEXTURE_SIZE;
+    texHeight = inContext->atlas.heigth;
+    texY = (atlasNode->position / inContext->atlasWidth);
+    texX = (atlasNode->position - texY*inContext->atlasWidth)*GLC_TEXTURE_SIZE;
+    texY *= GLC_TEXTURE_SIZE;
   }
   else {
     int factor = 0;
@@ -315,17 +370,17 @@ void __glcRenderCharTexture(__GLCfont* inFont, __GLCcontext* inContext,
     /* Try several texture size until we are able to create one */
     do {
       if (!__glcFaceDescGetBitmapSize(inFont->faceDesc, &pixWidth, &pixHeight,
-                                      boundingBox, scale_x, scale_y, factor,
-                                      inContext))
+                                      scale_x, scale_y, pixBoundingBox, factor,
+				      inContext))
 	return;
 
       factor = 1;
     } while (!__glcTextureGetImmediate(inContext, pixWidth, pixHeight));
 
     texWidth = inContext->texture.width;
-    texHeigth = inContext->texture.heigth;
-    posX = 0;
-    posY = 0;
+    texHeight = inContext->texture.heigth;
+    texX = 0;
+    texY = 0;
   }
 
   if (!inContext->texture.bufferObjectID || inContext->enableState.glObjects) {
@@ -373,7 +428,7 @@ void __glcRenderCharTexture(__GLCfont* inFont, __GLCcontext* inContext,
       pixBuffer = NULL;
     }
 
-    glTexSubImage2D(GL_TEXTURE_2D, level, posX >> level, posY >> level,
+    glTexSubImage2D(GL_TEXTURE_2D, level, texX >> level, texY >> level,
 		    pixWidth, pixHeight, GL_ALPHA, GL_UNSIGNED_BYTE,
 		    pixBuffer);
 
@@ -405,7 +460,7 @@ void __glcRenderCharTexture(__GLCfont* inFont, __GLCcontext* inContext,
       assert(!GLEW_ARB_pixel_buffer_object);
       memset(pixBuffer, 0, pixWidth * pixHeight);
       while ((pixWidth > 0) || (pixHeight > 0)) {
-	glTexSubImage2D(GL_TEXTURE_2D, level, posX >> level, posY >> level,
+	glTexSubImage2D(GL_TEXTURE_2D, level, texX >> level, texY >> level,
 		     pixWidth ? pixWidth : 1,
 		     pixHeight ? pixHeight : 1, GL_ALPHA,
 		     GL_UNSIGNED_BYTE, pixBuffer);
@@ -419,10 +474,6 @@ void __glcRenderCharTexture(__GLCfont* inFont, __GLCcontext* inContext,
 
   glPopClientAttrib();
 
-  /* Compute the size of the glyph */
-  width = (GLfloat)((boundingBox[2] - boundingBox[0]) / 64.);
-  heigth = (GLfloat)((boundingBox[3] - boundingBox[1]) / 64.);
-
   if (pixBuffer)
     __glcFree(pixBuffer);
 
@@ -435,8 +486,9 @@ void __glcRenderCharTexture(__GLCfont* inFont, __GLCcontext* inContext,
       GLfloat* data = NULL;
       __GLCatlasElement* atlasNode = inGlyph->textureObject;
 
-      buffer = __glcMalloc(inContext->atlasWidth * inContext->atlasHeight * 20
-			   * sizeof(GLfloat));
+      buffer = (GLfloat*)__glcMalloc(inContext->atlasWidth
+				     * inContext->atlasHeight * 20
+				     * sizeof(GLfloat));
       if (!buffer) {
 	__glcRaiseError(GLC_RESOURCE_ERROR);
 	return;
@@ -455,7 +507,7 @@ void __glcRenderCharTexture(__GLCfont* inFont, __GLCcontext* inContext,
        * VBO.
        */
       if (inContext->atlasCount > 1) {
-	data = glMapBufferARB(GL_ARRAY_BUFFER_ARB, GL_READ_ONLY);
+	data = (GLfloat*)glMapBufferARB(GL_ARRAY_BUFFER_ARB, GL_READ_ONLY);
 	if (!data) {
 	  __glcRaiseError(GLC_RESOURCE_ERROR);
 	  __glcFree(buffer);
@@ -467,20 +519,20 @@ void __glcRenderCharTexture(__GLCfont* inFont, __GLCcontext* inContext,
 
       data = buffer + atlasNode->position * 20;
 
-      data[0] = posX / texWidth;
-      data[1] = posY / texHeigth;
-      data[2] = boundingBox[0] / 64. / GLC_TEXTURE_SIZE;
-      data[3] = boundingBox[1] / 64. / GLC_TEXTURE_SIZE;
+      data[0] = texX / texWidth;
+      data[1] = texY / texHeight;
+      data[2] = pixBoundingBox[0] / 64. / GLC_TEXTURE_SIZE;
+      data[3] = pixBoundingBox[1] / 64. / GLC_TEXTURE_SIZE;
       data[4] = 0.f;
-      data[5] = (posX + width) / texWidth;
+      data[5] = (texX + GLC_TEXTURE_SIZE - 1) / texWidth;
       data[6] = data[1];
-      data[7] = boundingBox[2] / 64. / GLC_TEXTURE_SIZE;
+      data[7] = pixBoundingBox[2] / 64. / GLC_TEXTURE_SIZE;
       data[8] = data[3];
       data[9] = 0.f;
       data[10] = data[5];
-      data[11] = (posY + heigth) / texHeigth;
+      data[11] = (texY + GLC_TEXTURE_SIZE - 1) / texHeight;
       data[12] = data[7];
-      data[13] = boundingBox[3] / 64. / GLC_TEXTURE_SIZE;
+      data[13] = pixBoundingBox[3] / 64. / GLC_TEXTURE_SIZE;
       data[14] = 0.f;
       data[15] = data[0];
       data[16] = data[11];
@@ -514,27 +566,31 @@ void __glcRenderCharTexture(__GLCfont* inFont, __GLCcontext* inContext,
 
       /* Create the display list */
       glNewList(inGlyph->glObject[1], GL_COMPILE);
-      glScalef(1. / 64. / scale_x, 1. / 64. / scale_y , 1.);
+      glScalef(1. / (64. * scale_x), 1. / (64. * scale_y) , 1.);
 
       /* Modify the bouding box dimensions to compensate the glScalef() */
-      boundingBox[0] *= scale_x / GLC_TEXTURE_SIZE;
-      boundingBox[1] *= scale_y / GLC_TEXTURE_SIZE;
-      boundingBox[2] *= scale_x / GLC_TEXTURE_SIZE;
-      boundingBox[3] *= scale_y / GLC_TEXTURE_SIZE;
+      pixBoundingBox[0] *= scale_x / GLC_TEXTURE_SIZE;
+      pixBoundingBox[1] *= scale_y / GLC_TEXTURE_SIZE;
+      pixBoundingBox[2] *= scale_x / GLC_TEXTURE_SIZE;
+      pixBoundingBox[3] *= scale_y / GLC_TEXTURE_SIZE;
+
+      pixWidth = GLC_TEXTURE_SIZE;
+      pixHeight = GLC_TEXTURE_SIZE;
     }
   }
 
   /* Do the actual GL rendering */
   glBegin(GL_QUADS);
   glNormal3f(0.f, 0.f, 1.f);
-  glTexCoord2f(posX / texWidth, posY / texHeigth);
-  glVertex2i(boundingBox[0], boundingBox[1]);
-  glTexCoord2f((posX + width) / texWidth, posY / texHeigth);
-  glVertex2i(boundingBox[2], boundingBox[1]);
-  glTexCoord2f((posX + width) / texWidth, (posY + heigth) / texHeigth);
-  glVertex2i(boundingBox[2], boundingBox[3]);
-  glTexCoord2f(posX / texWidth, (posY + heigth) / texHeigth);
-  glVertex2i(boundingBox[0], boundingBox[3]);
+  glTexCoord2f(texX / texWidth, texY / texHeight);
+  glVertex2iv(pixBoundingBox);
+  glTexCoord2f((texX + pixWidth - 1) / texWidth, texY / texHeight);
+  glVertex2i(pixBoundingBox[2], pixBoundingBox[1]);
+  glTexCoord2f((texX + pixWidth - 1) / texWidth,
+	       (texY + pixHeight - 1) / texHeight);
+  glVertex2iv(pixBoundingBox + 2);
+  glTexCoord2f(texX / texWidth, (texY + pixHeight - 1) / texHeight);
+  glVertex2i(pixBoundingBox[0], pixBoundingBox[3]);
   glEnd();
 
   if (inContext->enableState.glObjects) {
