@@ -16,7 +16,7 @@
  *  License along with this library; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
-/* $Id: texture.h 707 2008-01-19 21:57:30Z bcoconni $ */
+/* $Id: texture.h 849 2008-11-02 13:02:23Z bcoconni $ */
 
 /** \file
  *  header of the routines used to render characters with textures.
@@ -34,7 +34,7 @@ struct __GLCatlasElementRec {
   __GLCglyph* glyph;
 };
 
-void __glcDeleteAtlasElement(__GLCatlasElement* This, __GLCcontext* inContext);
+void __glcReleaseAtlasElement(__GLCatlasElement* This, __GLCcontext* inContext);
 void __glcRenderCharTexture(__GLCfont* inFont, __GLCcontext* inContext,
 			    GLfloat scale_x, GLfloat scale_y,
 			    __GLCglyph* inGlyph);
