@@ -16,7 +16,7 @@
  *  License along with this library; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
-/* $Id: test7.c 700 2008-01-13 23:12:25Z bcoconni $ */
+/* $Id: test7.c 816 2008-08-01 23:54:22Z bcoconni $ */
 
 /** \file
  * Check that glcGetCharMetric() and glcGetStringCharMetric() return consistant
@@ -116,9 +116,9 @@ int main(int argc, char **argv) {
 
     CheckError();
 
-    for (j = 0; j < 4; j++) {
-      v1 = fabs(baseline1[j]);
-      v2 = fabs(baseline2[j]);
+    for (j = 0; j < 2; j++) {
+      v1 = fabs(baseline1[j + 2] - baseline1[j]);
+      v2 = fabs(baseline2[j + 2] - baseline2[j]);
       norm = v1 > v2 ? v1 : v2;
 
       if (fabs(v1 - v2) > EPSILON * norm) {
@@ -144,9 +144,9 @@ int main(int argc, char **argv) {
 
     CheckError();
 
-    for (j = 0; j < 8; j++) {
-      v1 = fabs(boundingBox1[j]);
-      v2 = fabs(boundingBox2[j]);
+    for (j = 0; j < 6; j++) {
+      v1 = fabs(boundingBox1[j + 2] - boundingBox1[j]);
+      v2 = fabs(boundingBox2[j + 2] - boundingBox2[j]);
       norm = v1 > v2 ? v1 : v2;
 
       if (fabs(v1 - v2) > EPSILON * norm) {
